@@ -20,19 +20,19 @@ import bpy
 from mathutils import (
         Matrix,
         Vector,
+        geometry,
         )
 from math import (
         sin, cos,
-        tan, radians,
+        tan, radians,atan,degrees
         )
 from random import triangular
+from bpy_extras.object_utils import AddObjectHelper, object_data_add
 
 NARROW_UI = 180
 MAX_INPUT_NUMBER = 50
 
-# Global_Scale = 0.001   # 1 blender unit = X mm
-GLOBAL_SCALE = 0.1       # 1 blender unit = X mm
-# Global_Scale = 1.0     # 1 blender unit = X mm
+GLOBAL_SCALE = 1       # 1 blender unit = X mm
 
 
 # next two utility functions are stolen from import_obj.py
@@ -67,9 +67,9 @@ def unpack_face_list(list_of_tuples):
 """
 Remove Doubles takes a list on Verts and a list of Faces and
 removes the doubles, much like Blender does in edit mode.
-It doesn’t have the range function  but it will round the corrdinates
-and remove verts that are very close togther.  The function
-is useful because you can perform a “Remove Doubles” with out
+It doesn't have the range function  but it will round the corrdinates
+and remove verts that are very close together.  The function
+is useful because you can perform a "Remove Doubles" with out
 having to enter Edit Mode. Having to enter edit mode has the
 disadvantage of not being able to interactively change the properties.
 """
@@ -168,7 +168,7 @@ def Get_Phillips_Bit_Height(Bit_Dia):
 # Returns a list of verts rotated by the given matrix. Used by SpinDup
 def Rot_Mesh(verts, matrix):
     from mathutils import Vector
-    return [(matrix * Vector(v))[:] for v in verts]
+    return [(matrix @ Vector(v))[:] for v in verts]
 
 
 # Returns a list of faces that has there index incremented by offset
@@ -232,20 +232,20 @@ def Mirror_Verts_Faces(VERTS, FACES, AXIS, FLIP_POINT=0):
 
 # Returns a list of faces that
 # make up an array of 4 point polygon.
-def Build_Face_List_Quads(OFFSET, COLUM, ROW, FLIP=0):
+def Build_Face_List_Quads(OFFSET, COLUMN, ROW, FLIP=0):
     Ret = []
     RowStart = 0
     for j in range(ROW):
-        for i in range(COLUM):
+        for i in range(COLUMN):
             Res1 = RowStart + i
-            Res2 = RowStart + i + (COLUM + 1)
-            Res3 = RowStart + i + (COLUM + 1) + 1
+            Res2 = RowStart + i + (COLUMN + 1)
+            Res3 = RowStart + i + (COLUMN + 1) + 1
             Res4 = RowStart + i + 1
             if FLIP:
                 Ret.append([OFFSET + Res1, OFFSET + Res2, OFFSET + Res3, OFFSET + Res4])
             else:
                 Ret.append([OFFSET + Res4, OFFSET + Res3, OFFSET + Res2, OFFSET + Res1])
-        RowStart += COLUM + 1
+        RowStart += COLUMN + 1
     return Ret
 
 
@@ -286,6 +286,29 @@ def Fill_Ring_Face(OFFSET, NUM, FACE_DOWN=0):
         Face[2] = TempFace[2]
     return Ret
 
+# Returns a list of faces that makes up a fill pattern around the last vert
+def Fill_Fan_Face(OFFSET, NUM, FACE_DOWN=0):
+    Ret = []
+    Face = [NUM-1,0,1]
+    TempFace = [0, 0, 0]
+    A = 0
+    #B = 1 unsed
+    C = 2
+    if NUM < 3:
+        return None
+    for _i in range(NUM - 2):
+        TempFace[0] = Face[A]
+        TempFace[1] = Face[C]
+        TempFace[2] = Face[C]+1
+        if FACE_DOWN:
+            Ret.append([OFFSET + Face[2], OFFSET + Face[1], OFFSET + Face[0]])
+        else:
+            Ret.append([OFFSET + Face[2], OFFSET + Face[1], OFFSET + Face[0]])
+
+        Face[0] = TempFace[0]
+        Face[1] = TempFace[1]
+        Face[2] = TempFace[2]
+    return Ret
 
 # ####################################################################
 #                    Create Allen Bit
@@ -379,7 +402,175 @@ def Create_Allen_Bit(FLAT_DISTANCE, HEIGHT):
     faces.extend(M_Faces)
 
     return verts, faces, OUTTER_RADIUS * 2.0
+# ####################################################################
+#                    Create Torx Bit
+# ####################################################################
 
+def Torx_Bit_Size_To_Point_Distance(Bit_Size):
+    if Bit_Size == 'bf_Torx_T10':
+        return 2.83
+    elif Bit_Size == 'bf_Torx_T20':
+        return 3.94
+    elif Bit_Size == 'bf_Torx_T25':
+        return 4.52
+    elif Bit_Size == 'bf_Torx_T30':
+        return 5.61
+    elif Bit_Size == 'bf_Torx_T40':
+        return 6.75
+    elif Bit_Size == 'bf_Torx_T50':
+        return 8.94
+    elif Bit_Size == 'bf_Torx_T55':
+        return 8.94
+    else:
+        return 2.83  #default to M3
+
+def Torx_Fill(OFFSET, FLIP=0):
+    faces = []
+    Lookup = [[0,10,11],
+            [0,11, 12],
+            [0,12,1],
+
+            [1, 12, 13],
+            [1, 13, 14],
+            [1, 14, 15],
+            [1, 15, 2],
+
+            [2, 15, 16],
+            [2, 16, 17],
+            [2, 17, 18],
+            [2, 18, 19],
+            [2, 19, 3],
+
+            [3, 19, 20],
+            [3, 20, 21],
+            [3, 21, 22],
+            [3, 22, 23],
+            [3, 23, 24],
+            [3, 24, 25],
+            [3, 25, 4],
+
+
+            [4, 25, 26],
+            [4, 26, 27],
+            [4, 27, 28],
+            [4, 28, 29],
+            [4, 29, 30],
+            [4, 30, 31],
+            [4, 31, 5],
+
+            [5, 31, 32],
+            [5, 32, 33],
+            [5, 33, 34],
+            [5, 34, 35],
+            [5, 35, 36],
+            [5, 36, 6],
+
+            [6, 36, 37],
+            [6, 37, 38],
+            [6, 38, 39],
+            [6, 39, 7],
+
+            [7, 39, 40],
+            [7, 40, 41],
+            [7, 41, 42],
+            [7, 42, 43],
+            [7, 43, 8],
+
+            [8, 43, 44],
+            [8, 44, 45],
+            [8, 45, 46],
+            [8, 46, 47],
+            [8, 47, 48],
+            [8, 48, 49],
+            [8, 49, 50],
+            [8, 50, 51],
+            [8, 51, 52],
+            [8, 52, 9],
+              ]
+    for i in Lookup:
+        if FLIP:
+            faces.append([OFFSET + i[2], OFFSET + i[1], OFFSET + i[0]])
+        else:
+            faces.append([OFFSET + i[0], OFFSET + i[1], OFFSET + i[2]])
+
+    return faces
+
+
+
+def Create_Torx_Bit(Point_Distance, HEIGHT):
+    verts = []
+    faces = []
+
+    POINT_RADIUS = Point_Distance * 0.5
+    OUTTER_RADIUS = POINT_RADIUS * 1.05
+
+    POINT_1_Y = POINT_RADIUS * 0.816592592592593
+    POINT_2_X = POINT_RADIUS * 0.511111111111111
+    POINT_2_Y = POINT_RADIUS * 0.885274074074074
+    POINT_3_X = POINT_RADIUS * 0.7072
+    POINT_3_Y = POINT_RADIUS * 0.408296296296296
+    POINT_4_X = POINT_RADIUS * 1.02222222222222
+    SMALL_RADIUS = POINT_RADIUS * 0.183407407407407
+    BIG_RADIUS = POINT_RADIUS * 0.333333333333333
+#     Values for T40    #     POINT_1_Y = 2.756
+#     POINT_2_X = 1.725
+#     POINT_2_Y = 2.9878
+#     POINT_3_X = 2.3868
+#     POINT_3_Y = 1.378
+#     POINT_4_X = 3.45
+#
+#     SMALL_RADIUS = 0.619
+#     BIG_RADIUS = 1.125
+
+    def Do_Curve(Curve_Height):
+        for i in range(0, 90, 10):
+            x = sin(radians(i)) * SMALL_RADIUS
+            y = cos(radians(i)) * SMALL_RADIUS
+            verts.append([x, POINT_1_Y + y, Curve_Height])
+
+        for i in range(260, 150, -10):
+            x = sin(radians(i)) * BIG_RADIUS
+            y = cos(radians(i)) * BIG_RADIUS
+            verts.append([POINT_2_X + x, POINT_2_Y + y, Curve_Height])
+
+        for i in range(340, 150 + 360, 10):
+            x = sin(radians(i%360)) * SMALL_RADIUS
+            y = cos(radians(i%360)) * SMALL_RADIUS
+            verts.append([POINT_3_X + x, POINT_3_Y + y, Curve_Height])
+
+        for i in range(320, 260, -10):
+            x = sin(radians(i)) * BIG_RADIUS
+            y = cos(radians(i)) * BIG_RADIUS
+            verts.append([POINT_4_X + x, y, Curve_Height])
+
+    FaceStart_Outside = len(verts)
+
+    for i in range(0, 100, 10):
+        x = sin(radians(i)) * OUTTER_RADIUS
+        y = cos(radians(i)) * OUTTER_RADIUS
+        verts.append([x, y, 0])
+
+    FaceStart_Top_Curve= len(verts)
+    Do_Curve(0)
+    faces.extend(Torx_Fill(FaceStart_Outside, 0))
+
+    FaceStart_Bottom_Curve= len(verts)
+    Do_Curve(0 - HEIGHT)
+
+    faces.extend(Build_Face_List_Quads(FaceStart_Top_Curve,42 ,1 , True))
+
+    verts.append([0,0,0 - HEIGHT]) # add center point for fill Fan
+    faces.extend(Fill_Fan_Face(FaceStart_Bottom_Curve, 44))
+
+    M_Verts, M_Faces = Mirror_Verts_Faces(verts, faces, 'x')
+    verts.extend(M_Verts)
+    faces.extend(M_Faces)
+
+    M_Verts, M_Faces = Mirror_Verts_Faces(verts, faces, 'y')
+    verts.extend(M_Verts)
+    faces.extend(M_Faces)
+
+    return verts, faces, OUTTER_RADIUS * 2.0
 
 # ####################################################################
 #                    Create Phillips Bit
@@ -627,7 +818,7 @@ def Create_CounterSink_Head(HOLE_DIA, HEAD_DIA, SHANK_DIA, HEIGHT, RAD1, DIV_COU
     sVerts, sFaces = SpinDup(verts, faces, 360, DIV_COUNT, 'z')
     sVerts.extend(verts)    # add the start verts to the Spin verts to complete the loop
 
-    faces.extend(Build_Face_List_Quads(FaceStart, Row - 1, DIV_COUNT, 1))
+    faces.extend(Build_Face_List_Quads(FaceStart, Row - 1, DIV_COUNT))
 
     return sVerts, faces, HEIGHT
 
@@ -882,6 +1073,280 @@ def Create_Hex_Head(FLAT, HOLE_DIA, SHANK_DIA, HEIGHT):
     return Spin_Verts, Spin_Faces, 0 - (-HEIGHT)
 
 
+
+def Create_12_Point(FLAT, HOLE_DIA, SHANK_DIA, HEIGHT,FLANGE_DIA):
+    FLANGE_HEIGHT = (1.89/8.0)*HEIGHT
+    FLAT_HEIGHT = (4.18/8.0)*HEIGHT
+#     FLANGE_DIA = (13.27/8.0)*FLAT
+
+    FLANGE_RADIUS = FLANGE_DIA * 0.5
+    FLANGE_TAPPER_HEIGHT =  HEIGHT - FLANGE_HEIGHT - FLAT_HEIGHT
+
+#     HOLE_DIA = 0.0
+
+    verts = []
+    faces = []
+    HOLE_RADIUS = HOLE_DIA / 2
+    Half_Flat = FLAT / 2
+    TopBevelRadius = Half_Flat - (Half_Flat * (0.05 / 8))
+#     Undercut_Height = (Half_Flat * (0.05 / 8))
+#     Shank_Bevel = (Half_Flat * (0.05 / 8))
+#     Flat_Height = HEIGHT - Undercut_Height - Shank_Bevel
+    # Undercut_Height = 5
+    SHANK_RADIUS = SHANK_DIA / 2
+    Row = 0
+
+    verts.append([0.0, 0.0, 0.0])
+
+#     print("HOLE_RADIUS" + str(HOLE_RADIUS))
+#     print("TopBevelRadius" + str(TopBevelRadius))
+
+    FaceStart = len(verts)
+
+    # inner hole
+    x = sin(radians(0)) * HOLE_RADIUS
+    y = cos(radians(0)) * HOLE_RADIUS
+    verts.append([x, y, 0.0])
+
+    x = sin(radians(5)) * HOLE_RADIUS
+    y = cos(radians(5)) * HOLE_RADIUS
+    verts.append([x, y, 0.0])
+
+    x = sin(radians(10)) * HOLE_RADIUS
+    y = cos(radians(10)) * HOLE_RADIUS
+    verts.append([x, y, 0.0])
+
+    x = sin(radians(15)) * HOLE_RADIUS
+    y = cos(radians(15)) * HOLE_RADIUS
+    verts.append([x, y, 0.0])
+
+    x = sin(radians(20)) * HOLE_RADIUS
+    y = cos(radians(20)) * HOLE_RADIUS
+    verts.append([x, y, 0.0])
+
+    x = sin(radians(25)) * HOLE_RADIUS
+    y = cos(radians(25)) * HOLE_RADIUS
+    verts.append([x, y, 0.0])
+
+    x = sin(radians(30)) * HOLE_RADIUS
+    y = cos(radians(30)) * HOLE_RADIUS
+    verts.append([x, y, 0.0])
+
+    Row += 1
+
+
+
+    # bevel
+    x = sin(radians(0)) * TopBevelRadius
+    y = cos(radians(0)) * TopBevelRadius
+    vec1 = Vector([x, y, 0.0])
+    verts.append([x, y, 0.0])
+
+    x = sin(radians(5)) * TopBevelRadius
+    y = cos(radians(5)) * TopBevelRadius
+    vec2 = Vector([x, y, 0.0])
+    verts.append([x, y, 0.0])
+
+    x = sin(radians(10)) * TopBevelRadius
+    y = cos(radians(10)) * TopBevelRadius
+    vec3 = Vector([x, y, 0.0])
+    verts.append([x, y, 0.0])
+
+    x = sin(radians(15)) * TopBevelRadius
+    y = cos(radians(15)) * TopBevelRadius
+    vec4 = Vector([x, y, 0.0])
+    verts.append([x, y, 0.0])
+
+    x = sin(radians(20)) * TopBevelRadius
+    y = cos(radians(20)) * TopBevelRadius
+    vec5 = Vector([x, y, 0.0])
+    verts.append([x, y, 0.0])
+
+    x = sin(radians(25)) * TopBevelRadius
+    y = cos(radians(25)) * TopBevelRadius
+    vec6 = Vector([x, y, 0.0])
+    verts.append([x, y, 0.0])
+
+    x = sin(radians(30)) * TopBevelRadius
+    y = cos(radians(30)) * TopBevelRadius
+    vec7 = Vector([x, y, 0.0])
+    verts.append([x, y, 0.0])
+
+    Row += 1
+
+
+    #45Deg bevel on the top
+
+    #First we work out how far up the Y axis the vert is
+    v_origin = Vector([0.0,0.0,0.0]) # center of the model
+    v_15Deg_Point = Vector([tan(radians(15)) * Half_Flat,Half_Flat,0.0])  #Is a know point to work back from
+
+    x = tan(radians(0)) * Half_Flat
+    Point_Distance =(tan(radians(30)) * v_15Deg_Point.x)+Half_Flat
+    dvec = vec1 - Vector([x, Point_Distance, 0.0])
+    verts.append([x, Point_Distance, -dvec.length])
+    v_0_Deg_Top_Point = Vector([x, Point_Distance, -dvec.length])
+
+    v_0_Deg_Point = Vector([x, Point_Distance,0.0])
+
+    v_5Deg_Line = Vector([tan(radians(5)) * Half_Flat, Half_Flat, 0.0])
+    v_5Deg_Line.length *= 2  # extende out the line on a 5 deg angle
+
+    #We cross 2 lines. One from the origin to the 0 Deg point
+    #and the second is from the orign extended out past the first line
+    # This gives the cross point of the
+    v_Cross = geometry.intersect_line_line_2d(v_0_Deg_Point,v_15Deg_Point,v_origin,v_5Deg_Line)
+    dvec = vec2 - Vector([v_Cross.x,v_Cross.y,0.0])
+    verts.append([v_Cross.x,v_Cross.y,-dvec.length])
+    v_5_Deg_Top_Point = Vector([v_Cross.x,v_Cross.y,-dvec.length])
+
+    v_10Deg_Line = Vector([tan(radians(10)) * Half_Flat, Half_Flat, 0.0])
+    v_10Deg_Line.length *= 2  # extende out the line
+
+    v_Cross = geometry.intersect_line_line_2d(v_0_Deg_Point,v_15Deg_Point,v_origin,v_10Deg_Line)
+    dvec = vec3 - Vector([v_Cross.x,v_Cross.y,0.0])
+    verts.append([v_Cross.x,v_Cross.y,-dvec.length])
+    v_10_Deg_Top_Point = Vector([v_Cross.x,v_Cross.y,-dvec.length])
+
+    #The remain points are stright forward because y is all the same y height (Half_Flat)
+    x = tan(radians(15)) * Half_Flat
+    dvec = vec4 - Vector([x, Half_Flat, 0.0])
+    Lowest_Point = -dvec.length
+    verts.append([x, Half_Flat, -dvec.length])
+    v_15_Deg_Top_Point = Vector([x, Half_Flat, -dvec.length])
+
+    x = tan(radians(20)) * Half_Flat
+    dvec = vec5 - Vector([x, Half_Flat, 0.0])
+    Lowest_Point = -dvec.length
+    verts.append([x, Half_Flat, -dvec.length])
+    v_20_Deg_Top_Point = Vector([x, Half_Flat, -dvec.length])
+
+    x = tan(radians(25)) * Half_Flat
+    dvec = vec6 - Vector([x, Half_Flat, 0.0])
+    Lowest_Point = -dvec.length
+    verts.append([x, Half_Flat, -dvec.length])
+    v_25_Deg_Top_Point = Vector([x, Half_Flat, -dvec.length])
+
+    x = tan(radians(30)) * Half_Flat
+    dvec = vec7 - Vector([x, Half_Flat, 0.0])
+    Lowest_Point = -dvec.length
+    verts.append([x, Half_Flat, -dvec.length])
+    v_30_Deg_Top_Point = Vector([x, Half_Flat, -dvec.length])
+    Row += 1
+
+
+    #Down Bits
+#     print ("Point_Distance")
+#     print (Point_Distance)
+
+
+
+    Flange_Adjacent  = FLANGE_RADIUS - Point_Distance
+    if (Flange_Adjacent  == 0.0):
+        Flange_Adjacent  = 0.000001
+    Flange_Opposite = FLANGE_TAPPER_HEIGHT
+
+#     print ("Flange_Opposite")
+#     print (Flange_Opposite)
+#     print ("Flange_Adjacent")
+#     print (Flange_Adjacent)
+
+    FLANGE_ANGLE_RAD = atan(Flange_Opposite/Flange_Adjacent )
+#     FLANGE_ANGLE_RAD = radians(45)
+#     print("FLANGE_ANGLE_RAD")
+#     print (degrees (FLANGE_ANGLE_RAD))
+
+
+    v_Extended_Flange_Edge = Vector([0.0,0.0,-HEIGHT + FLANGE_HEIGHT + (tan(FLANGE_ANGLE_RAD)* FLANGE_RADIUS) ])
+#     print("v_Extended_Flange_Edge")
+#     print (v_Extended_Flange_Edge)
+
+    #0deg
+    v_Flange_Edge = Vector([sin(radians(0)) * FLANGE_RADIUS,cos(radians(0)) * FLANGE_RADIUS,-HEIGHT + FLANGE_HEIGHT ])
+    v_Cross = geometry.intersect_line_line(v_0_Deg_Top_Point,Vector([v_0_Deg_Top_Point.x,v_0_Deg_Top_Point.y,-HEIGHT]),v_Flange_Edge,v_Extended_Flange_Edge)
+    verts.append(v_Cross[0])
+
+    #5deg
+    v_Flange_Edge = Vector([sin(radians(5)) * FLANGE_RADIUS,cos(radians(5)) * FLANGE_RADIUS,-HEIGHT + FLANGE_HEIGHT ])
+    v_Cross = geometry.intersect_line_line(v_5_Deg_Top_Point,Vector([v_5_Deg_Top_Point.x,v_5_Deg_Top_Point.y,-HEIGHT]),v_Flange_Edge,v_Extended_Flange_Edge)
+    verts.append(v_Cross[0])
+
+    #10deg
+    v_Flange_Edge = Vector([sin(radians(10)) * FLANGE_RADIUS,cos(radians(10)) * FLANGE_RADIUS,-HEIGHT + FLANGE_HEIGHT ])
+    v_Cross = geometry.intersect_line_line(v_10_Deg_Top_Point,Vector([v_10_Deg_Top_Point.x,v_10_Deg_Top_Point.y,-HEIGHT]),v_Flange_Edge,v_Extended_Flange_Edge)
+    verts.append(v_Cross[0])
+
+    #15deg
+    v_Flange_Edge = Vector([sin(radians(15)) * FLANGE_RADIUS,cos(radians(15)) * FLANGE_RADIUS,-HEIGHT + FLANGE_HEIGHT ])
+    v_Cross = geometry.intersect_line_line(v_15_Deg_Top_Point,Vector([v_15_Deg_Top_Point.x,v_15_Deg_Top_Point.y,-HEIGHT]),v_Flange_Edge,v_Extended_Flange_Edge)
+    verts.append(v_Cross[0])
+
+
+    #20deg
+    v_Flange_Edge = Vector([sin(radians(20)) * FLANGE_RADIUS,cos(radians(20)) * FLANGE_RADIUS,-HEIGHT + FLANGE_HEIGHT ])
+    v_Cross = geometry.intersect_line_line(v_20_Deg_Top_Point,Vector([v_20_Deg_Top_Point.x,v_20_Deg_Top_Point.y,-HEIGHT]),v_Flange_Edge,v_Extended_Flange_Edge)
+    verts.append(v_Cross[0])
+
+    #25deg
+    v_Flange_Edge = Vector([sin(radians(25)) * FLANGE_RADIUS,cos(radians(25)) * FLANGE_RADIUS,-HEIGHT + FLANGE_HEIGHT ])
+    v_Cross = geometry.intersect_line_line(v_25_Deg_Top_Point,Vector([v_25_Deg_Top_Point.x,v_25_Deg_Top_Point.y,-HEIGHT]),v_Flange_Edge,v_Extended_Flange_Edge)
+    verts.append(v_Cross[0])
+
+
+    #30deg
+    v_Flange_Edge = Vector([sin(radians(30)) * FLANGE_RADIUS,cos(radians(30)) * FLANGE_RADIUS,-HEIGHT + FLANGE_HEIGHT ])
+    v_Cross = geometry.intersect_line_line(v_30_Deg_Top_Point,Vector([v_30_Deg_Top_Point.x,v_30_Deg_Top_Point.y,-HEIGHT]),v_Flange_Edge,v_Extended_Flange_Edge)
+    verts.append(v_Cross[0])
+
+    Row += 1
+
+
+
+    verts.append([sin(radians(0)) * FLANGE_RADIUS,cos(radians(0)) * FLANGE_RADIUS,-HEIGHT + FLANGE_HEIGHT ])
+    verts.append([sin(radians(5)) * FLANGE_RADIUS,cos(radians(5)) * FLANGE_RADIUS,-HEIGHT + FLANGE_HEIGHT])
+    verts.append([sin(radians(10)) * FLANGE_RADIUS,cos(radians(10)) * FLANGE_RADIUS,-HEIGHT + FLANGE_HEIGHT])
+    verts.append([sin(radians(15)) * FLANGE_RADIUS,cos(radians(15)) * FLANGE_RADIUS,-HEIGHT + FLANGE_HEIGHT])
+    verts.append([sin(radians(20)) * FLANGE_RADIUS,cos(radians(20)) * FLANGE_RADIUS,-HEIGHT + FLANGE_HEIGHT])
+    verts.append([sin(radians(25)) * FLANGE_RADIUS,cos(radians(25)) * FLANGE_RADIUS,-HEIGHT + FLANGE_HEIGHT])
+    verts.append([sin(radians(30)) * FLANGE_RADIUS,cos(radians(30)) * FLANGE_RADIUS,-HEIGHT + FLANGE_HEIGHT])
+
+    Row += 1
+
+    verts.append([sin(radians(0)) * FLANGE_RADIUS,cos(radians(0)) * FLANGE_RADIUS,-HEIGHT])
+    verts.append([sin(radians(5)) * FLANGE_RADIUS,cos(radians(5)) * FLANGE_RADIUS,-HEIGHT])
+    verts.append([sin(radians(10)) * FLANGE_RADIUS,cos(radians(10)) * FLANGE_RADIUS,-HEIGHT])
+    verts.append([sin(radians(15)) * FLANGE_RADIUS,cos(radians(15)) * FLANGE_RADIUS,-HEIGHT])
+    verts.append([sin(radians(20)) * FLANGE_RADIUS,cos(radians(20)) * FLANGE_RADIUS,-HEIGHT])
+    verts.append([sin(radians(25)) * FLANGE_RADIUS,cos(radians(25)) * FLANGE_RADIUS,-HEIGHT])
+    verts.append([sin(radians(30)) * FLANGE_RADIUS,cos(radians(30)) * FLANGE_RADIUS,-HEIGHT])
+
+    Row += 1
+
+
+    verts.append([sin(radians(0)) * SHANK_RADIUS,cos(radians(0)) * SHANK_RADIUS,-HEIGHT])
+    verts.append([sin(radians(0)) * SHANK_RADIUS,cos(radians(0)) * SHANK_RADIUS,-HEIGHT])
+    verts.append([sin(radians(10)) * SHANK_RADIUS,cos(radians(10)) * SHANK_RADIUS,-HEIGHT])
+    verts.append([sin(radians(10)) * SHANK_RADIUS,cos(radians(10)) * SHANK_RADIUS,-HEIGHT])
+    verts.append([sin(radians(20)) * SHANK_RADIUS,cos(radians(20)) * SHANK_RADIUS,-HEIGHT])
+    verts.append([sin(radians(20)) * SHANK_RADIUS,cos(radians(20)) * SHANK_RADIUS,-HEIGHT])
+    verts.append([sin(radians(30)) * SHANK_RADIUS,cos(radians(30)) * SHANK_RADIUS,-HEIGHT])
+
+    Row += 1
+
+
+    faces.extend(Build_Face_List_Quads(FaceStart, 6, Row - 1))
+
+    Spin_Verts, Spin_Faces = SpinDup(verts, faces, 360,12, 'z')
+
+    return Spin_Verts, Spin_Faces, 0 - (-HEIGHT)
+
+
+def Create_12_Point_Head(FLAT, HOLE_DIA, SHANK_DIA, HEIGHT,FLANGE_DIA):
+    #TODO add under head radius
+    return  Create_12_Point(FLAT, HOLE_DIA, SHANK_DIA, HEIGHT,FLANGE_DIA)
+
+
+
 # ####################################################################
 #                    Create External Thread
 # ####################################################################
@@ -1481,7 +1946,7 @@ def add_Nylon_Head(OUTSIDE_RADIUS, Z_LOCATION, DIV_COUNT):
     INNER_HOLE = OUTSIDE_RADIUS - (OUTSIDE_RADIUS * (1.25 / 4.75))
     EDGE_THICKNESS = (OUTSIDE_RADIUS * (0.4 / 4.75))
     RAD1 = (OUTSIDE_RADIUS * (0.5 / 4.75))
-    OVER_ALL_HEIGTH = (OUTSIDE_RADIUS * (2.0 / 4.75))
+    OVER_ALL_HEIGHT = (OUTSIDE_RADIUS * (2.0 / 4.75))
 
     FaceStart = len(verts)
 
@@ -1490,13 +1955,13 @@ def add_Nylon_Head(OUTSIDE_RADIUS, Z_LOCATION, DIV_COUNT):
     Lowest_Z_Vert = 0
 
     x = INNER_HOLE
-    z = (Height_Offset - OVER_ALL_HEIGTH) + EDGE_THICKNESS
+    z = (Height_Offset - OVER_ALL_HEIGHT) + EDGE_THICKNESS
     verts.append([x, 0.0, z])
     Lowest_Z_Vert = min(Lowest_Z_Vert, z)
     Row += 1
 
     x = INNER_HOLE
-    z = (Height_Offset - OVER_ALL_HEIGTH)
+    z = (Height_Offset - OVER_ALL_HEIGHT)
     verts.append([x, 0.0, z])
     Lowest_Z_Vert = min(Lowest_Z_Vert, z)
     Row += 1
@@ -1504,7 +1969,7 @@ def add_Nylon_Head(OUTSIDE_RADIUS, Z_LOCATION, DIV_COUNT):
     for i in range(180, 80, -10):
         x = sin(radians(i)) * RAD1
         z = cos(radians(i)) * RAD1
-        verts.append([(OUTSIDE_RADIUS - RAD1) + x, 0.0, ((Height_Offset - OVER_ALL_HEIGTH) + RAD1) + z])
+        verts.append([(OUTSIDE_RADIUS - RAD1) + x, 0.0, ((Height_Offset - OVER_ALL_HEIGHT) + RAD1) + z])
         Lowest_Z_Vert = min(Lowest_Z_Vert, z)
         Row += 1
 
@@ -1517,7 +1982,7 @@ def add_Nylon_Head(OUTSIDE_RADIUS, Z_LOCATION, DIV_COUNT):
     sVerts, sFaces = SpinDup(verts, faces, 360, DIV_COUNT, 'z')
     sVerts.extend(verts)        # add the start verts to the Spin verts to complete the loop
 
-    faces.extend(Build_Face_List_Quads(FaceStart, Row - 1, DIV_COUNT))
+    faces.extend(Build_Face_List_Quads(FaceStart, Row - 1, DIV_COUNT,1))
 
     return Move_Verts_Up_Z(sVerts, 0), faces, Lowest_Z_Vert
 
@@ -1529,8 +1994,8 @@ def add_Nylon_Part(OUTSIDE_RADIUS, Z_LOCATION, DIV_COUNT):
 
     INNER_HOLE = OUTSIDE_RADIUS - (OUTSIDE_RADIUS * (1.5 / 4.75))
     EDGE_THICKNESS = (OUTSIDE_RADIUS * (0.4 / 4.75))
-    OVER_ALL_HEIGTH = (OUTSIDE_RADIUS * (2.0 / 4.75))
-    PART_THICKNESS = OVER_ALL_HEIGTH - EDGE_THICKNESS
+    OVER_ALL_HEIGHT = (OUTSIDE_RADIUS * (2.0 / 4.75))
+    PART_THICKNESS = OVER_ALL_HEIGHT - EDGE_THICKNESS
     PART_INNER_HOLE = (OUTSIDE_RADIUS * (2.5 / 4.75))
 
     FaceStart = len(verts)
@@ -1570,6 +2035,11 @@ def add_Nylon_Part(OUTSIDE_RADIUS, Z_LOCATION, DIV_COUNT):
     return sVerts, faces, 0 - Lowest_Z_Vert
 
 
+def add_12_Point_Nut(FLAT, HOLE_DIA, HEIGHT,FLANGE_DIA):
+    return Create_12_Point(FLAT, HOLE_DIA,HOLE_DIA, HEIGHT,FLANGE_DIA)
+
+
+
 # ####################################################################
 #                   Create Internal Thread
 # ####################################################################
@@ -1816,12 +2286,18 @@ def Nut_Mesh(props, context):
     Head_Verts = []
     Head_Faces = []
 
-    New_Nut_Height = 5
 
     Face_Start = len(verts)
+
+
+    if props.bf_Nut_Type == 'bf_Nut_12Pnt':
+        Nut_Height = props.bf_12_Point_Nut_Height
+    else:
+        Nut_Height = props.bf_Hex_Nut_Height
+
     Thread_Verts, Thread_Faces, New_Nut_Height = Create_Internal_Thread(
                                                     props.bf_Minor_Dia, props.bf_Major_Dia,
-                                                    props.bf_Pitch, props.bf_Hex_Nut_Height,
+                                                    props.bf_Pitch, Nut_Height,
                                                     props.bf_Crest_Percent, props.bf_Root_Percent,
                                                     1, props.bf_Div_Count
                                                     )
@@ -1829,7 +2305,16 @@ def Nut_Mesh(props, context):
     faces.extend(Copy_Faces(Thread_Faces, Face_Start))
 
     Face_Start = len(verts)
-    Head_Verts, Head_Faces, Lock_Nut_Rad = add_Hex_Nut(
+
+    if props.bf_Nut_Type == 'bf_Nut_12Pnt':
+        Head_Verts, Head_Faces, Lock_Nut_Rad = add_12_Point_Nut(
+                                                props.bf_12_Point_Nut_Flat_Distance,
+                                                props.bf_Major_Dia, New_Nut_Height,
+                                                #Limit the size of the Flange to avoid calculation error
+                                                max(props.bf_12_Point_Nut_Flange_Dia,props.bf_12_Point_Nut_Flat_Distance)
+                                                )
+    else:
+        Head_Verts, Head_Faces, Lock_Nut_Rad = add_Hex_Nut(
                                                 props.bf_Hex_Nut_Flat_Distance,
                                                 props.bf_Major_Dia, New_Nut_Height
                                                 )
@@ -1882,7 +2367,8 @@ def Bolt_Mesh(props, context):
         if Allen_Bit_Dia(props.bf_Allen_Bit_Flat_Distance) > Max_Pan_Bit_Dia(props.bf_Pan_Head_Dia):
             ReSized_Allen_Bit_Flat_Distance = Allen_Bit_Dia_To_Flat(
                                                 Max_Pan_Bit_Dia(props.bf_Pan_Head_Dia)
-                                                ) * 1.05
+                                                )
+            ReSized_Allen_Bit_Flat_Distance -= ReSized_Allen_Bit_Flat_Distance * 0.05   # It looks better if it is just a bit smaller
             # print ("Resized Allen Bit Flat Distance to ",ReSized_Allen_Bit_Flat_Distance)
 
     # Bit Mesh
@@ -1892,6 +2378,13 @@ def Bolt_Mesh(props, context):
                                                 props.bf_Allen_Bit_Depth
                                                 )
 
+    if props.bf_Bit_Type == 'bf_Bit_Torx':
+        Bit_Verts, Bit_Faces, Bit_Dia = Create_Torx_Bit(
+                                                Torx_Bit_Size_To_Point_Distance(props.bf_Torx_Size_Type),
+                                                props.bf_Torx_Bit_Depth
+                                                )
+
+
     if props.bf_Bit_Type == 'bf_Bit_Philips':
         Bit_Verts, Bit_Faces, Bit_Dia = Create_Phillips_Bit(
                                                 props.bf_Philips_Bit_Dia,
@@ -1904,6 +2397,14 @@ def Bolt_Mesh(props, context):
                                                 props.bf_Hex_Head_Flat_Distance, Bit_Dia,
                                                 props.bf_Shank_Dia, props.bf_Hex_Head_Height
                                                 )
+
+    elif props.bf_Head_Type == 'bf_Head_12Pnt':
+        Head_Verts, Head_Faces, Head_Height = Create_12_Point_Head(
+                                                props.bf_12_Point_Head_Flat_Distance, Bit_Dia,
+                                                props.bf_Shank_Dia, props.bf_12_Point_Head_Height,
+                                                #Limit the size of the Flange to avoid calculation error
+                                                max(props.bf_12_Point_Head_Flange_Dia,props.bf_12_Point_Head_Flat_Distance)
+                                                )
     elif props.bf_Head_Type == 'bf_Head_Cap':
         Head_Verts, Head_Faces, Head_Height = Create_Cap_Head(
                                                 Bit_Dia, props.bf_Cap_Head_Dia,
@@ -1933,14 +2434,7 @@ def Bolt_Mesh(props, context):
                                                 props.bf_CounterSink_Head_Dia * (0.09 / 6.31),
                                                 props.bf_Div_Count
                                                 )
-    """
-    Head_Verts, Head_Faces, Head_Height = Create_CounterSink_Head(
-                                                Bit_Dia, props.bf_CounterSink_Head_Dia,
-                                                props.bf_Shank_Dia,
-                                                props.bf_CounterSink_Head_Dia,
-                                                props.bf_CounterSink_Head_Dia * (1.0 / 19.0)
-                                                )
-    """
+
     Face_Start = len(verts)
     verts.extend(Move_Verts_Up_Z(Bit_Verts, Head_Height))
     faces.extend(Copy_Faces(Bit_Faces, Face_Start))
@@ -1958,146 +2452,48 @@ def Bolt_Mesh(props, context):
                                                     props.bf_Root_Percent, props.bf_Div_Count
                                                     )
 
-    verts.extend(Move_Verts_Up_Z(Thread_Verts, 00))
+    verts.extend(Move_Verts_Up_Z(Thread_Verts, 0))
     faces.extend(Copy_Faces(Thread_Faces, Face_Start))
 
     return Move_Verts_Up_Z(verts, Thread_Height), faces
 
 
-# calculates the matrix for the new object
-# depending on user pref
-def align_matrix(context):
-    loc = Matrix.Translation(context.scene.cursor_location)
-    obj_align = context.user_preferences.edit.object_align
-    if (context.space_data.type == 'VIEW_3D' and obj_align == 'VIEW'):
-        rot = context.space_data.region_3d.view_matrix.to_3x3().inverted().to_4x4()
-    else:
-        rot = Matrix()
-    align_matrix = loc * rot
-    return align_matrix
 
 
-# Create a new mesh (object) from verts/edges/faces.
-# verts/edges/faces ... List of vertices/edges/faces for the
-#                       new mesh (as used in from_pydata).
-# name ... Name of the new mesh (& object).
-# edit ... Replace existing mesh data.
-# Note: Using "edit" will destroy/delete existing mesh data.
-def create_mesh_object(context, verts, edges, faces, name, edit, align_matrix):
-    scene = context.scene
-    obj_act = scene.objects.active
-
-    # Can't edit anything, unless we have an active obj.
-    if edit and not obj_act:
-        return None
-
-    # Create new mesh
-    mesh = bpy.data.meshes.new(name)
-
-    # Make a mesh from a list of verts/edges/faces.
-    mesh.from_pydata(verts, edges, faces)
-
-    # Fix T51338 : Validate the mesh (the internal thread generator for the Nut
-    # should be more reliable now, however there could be other possible errors)
-    is_not_mesh_valid = mesh.validate()
-
-    if is_not_mesh_valid:
-        print("\n[BoltFactory]\nFunction: create_mesh_object\n"
-              "Mesh is not Valid, correcting\n")
-
-    # Update mesh geometry after adding stuff.
-    mesh.update()
-
-    # Deselect all objects when in object mode
-    if bpy.ops.object.select_all.poll():
-        bpy.ops.object.select_all(action='DESELECT')
-
-    if edit:
-        # Replace geometry of existing object
-
-        # Use the active obj and select it.
-        ob_new = obj_act
-        ob_new.select = True
-
-        if obj_act.mode == 'OBJECT':
-            # Get existing mesh datablock.
-            old_mesh = ob_new.data
-
-            # Set object data to nothing
-            ob_new.data = None
-
-            # Clear users of existing mesh datablock.
-            old_mesh.user_clear()
-
-            # Remove old mesh datablock if no users are left.
-            if (old_mesh.users == 0):
-                bpy.data.meshes.remove(old_mesh)
-
-            # Assign new mesh datablock.
-            ob_new.data = mesh
-    else:
-        # Create new object
-        ob_new = bpy.data.objects.new(name, mesh)
-
-        # Link new object to the given scene and select it.
-        scene.objects.link(ob_new)
-        ob_new.select = True
-
-        # Place the object at the 3D cursor location.
-        # apply viewRotaion
-        ob_new.matrix_world = align_matrix
-
-    if obj_act and obj_act.mode == 'EDIT':
-        if not edit:
-            # We are in EditMode, switch to ObjectMode.
-            bpy.ops.object.mode_set(mode='OBJECT')
-
-            # Select the active object as well.
-            obj_act.select = True
-
-            # Apply location of new object.
-            scene.update()
-
-            # Join new object into the active.
-            bpy.ops.object.join()
-
-            # Switching back to EditMode.
-            bpy.ops.object.mode_set(mode='EDIT')
-
-            ob_new = obj_act
-
-    else:
-        # We are in ObjectMode.
-        # Make the new object the active one.
-        scene.objects.active = ob_new
-
-    return ob_new
 
 
-def Create_New_Mesh(props, context, align_matrix):
+
+
+def Create_New_Mesh(props, context):
 
     verts = []
     faces = []
-    # sMeshName =''  # UNUSED
+    edges = []
     sObjName = ''
 
     if props.bf_Model_Type == 'bf_Model_Bolt':
         # print('Create Bolt')
         verts, faces = Bolt_Mesh(props, context)
-        # sMeshName = 'Bolt'  # UNUSED
         sObjName = 'Bolt'
 
     if props.bf_Model_Type == 'bf_Model_Nut':
         # print('Create Nut')
         verts, faces = Nut_Mesh(props, context)
-        # sMeshName = 'Nut'  # UNUSED
         sObjName = 'Nut'
 
     verts, faces = RemoveDoubles(verts, faces)
 
     verts = Scale_Mesh_Verts(verts, GLOBAL_SCALE)
 
-    obj = create_mesh_object(context, verts, [], faces, sObjName,
-                             props.edit, align_matrix)
+    mesh = bpy.data.meshes.new(name=sObjName)
+    mesh.from_pydata(verts, edges, faces)
 
-    return obj
+    # useful for development when the mesh may be invalid.
+    # Fix T51338 : Validate the mesh (the internal thread generator for the Nut
+    # should be more reliable now, however there could be other possible errors)
+    is_not_mesh_valid = mesh.validate()
+
+    if is_not_mesh_valid:
+        props.report({'INFO'}, "Mesh is not Valid, correcting")
+
+    return mesh
