@@ -31,12 +31,40 @@ def region_to_location(viewcoords, depthcoords):
     from bpy_extras import view3d_utils
     return view3d_utils.region_2d_to_location_3d(bpy.context.region, bpy.context.space_data.region_3d, viewcoords, depthcoords)
 
-def assign_vg(obj, vg_name):
+def store_cage(self, vg_name):
+    import time
+    unique_id = time.strftime(r'%y%m%d%H%M%S') # ex: 20210711111117
+    # name = f'gp_lattice_{unique_id}'
+    name = f'{self.gp_obj.name}_lat{unique_id}'
+    vg = self.gp_obj.vertex_groups.get(vg_name)
+    if vg:
+        vg.name = name
+    for o in self.other_gp:
+        vg = o.vertex_groups.get(vg_name)
+        if vg:
+            vg.name = name
+
+    self.cage.name = name
+    self.cage.data.name = name
+    mod = self.gp_obj.grease_pencil_modifiers.get('tmp_lattice')
+    if mod:
+        mod.name = name #f'Lattice_{unique_id}'
+        mod.vertex_group = name
+    for o in self.other_gp:
+        mod = o.grease_pencil_modifiers.get('tmp_lattice')
+        if mod:
+            mod.name = name
+            mod.vertex_group = name
+
+def assign_vg(obj, vg_name, delete=False):
     ## create vertex group
     vg = obj.vertex_groups.get(vg_name)
     if vg:
         # remove to start clean
         obj.vertex_groups.remove(vg)
+    if delete:
+        return
+
     vg = obj.vertex_groups.new(name=vg_name)
     bpy.ops.gpencil.vertex_group_assign()
     return vg
@@ -48,6 +76,10 @@ def view_cage(obj):
     gp = obj.data
     gpl = gp.layers
 
+    from_obj = bpy.context.mode == 'OBJECT'
+    all_gps = [o for o in bpy.context.selected_objects if o.type == 'GPENCIL']
+    other_gp = [o for o in all_gps if o is not obj]
+
     coords = []
     initial_mode = bpy.context.mode
 
@@ -60,7 +92,7 @@ def view_cage(obj):
                 target_frames = [f for f in l.frames if f.select]
             else:
                 target_frames = [l.active_frame]
-            
+
             for f in target_frames:
                 for s in f.strokes:
                     if not s.select:
@@ -70,14 +102,15 @@ def view_cage(obj):
                             # get real location
                             coords.append(obj.matrix_world @ p.co)
 
-    elif bpy.context.mode == 'OBJECT':#object mode -> all points
-        for l in gpl:# if l.hide:continue# only visible ? (might break things)
-            if not len(l.frames):
-                continue#skip frameless layer
-            for s in l.active_frame.strokes:
-                for p in s.points:
-                    coords.append(obj.matrix_world @ p.co)
-    
+    elif bpy.context.mode == 'OBJECT': # object mode -> all points of all selected gp objects
+        for gpo in all_gps:
+            for l in gpo.data.layers:# if l.hide:continue# only visible ? (might break things)
+                if not len(l.frames):
+                    continue # skip frameless layer
+                for s in l.active_frame.strokes:
+                    for p in s.points:
+                        coords.append(gpo.matrix_world @ p.co)
+
     elif bpy.context.mode == 'PAINT_GPENCIL':
         # get last stroke points coordinated
         if not gpl.active or not gpl.active.active_frame:
@@ -85,12 +118,12 @@ def view_cage(obj):
 
         if not len(gpl.active.active_frame.strokes):
             return 'No stroke found to deform'
-        
+
         paint_id = -1
         if bpy.context.scene.tool_settings.use_gpencil_draw_onback:
             paint_id = 0
         coords = [obj.matrix_world @ p.co for p in gpl.active.active_frame.strokes[paint_id].points]
-    
+
     else:
         return 'Wrong mode!'
 
@@ -106,24 +139,24 @@ def view_cage(obj):
 
     if bpy.context.mode == 'EDIT_GPENCIL':
         vg = assign_vg(obj, vg_name)
-    
+
     if bpy.context.mode == 'PAINT_GPENCIL':
         # points cannot be assign to API yet(ugly and slow workaround but only way)
         # -> https://developer.blender.org/T56280 so, hop'in'ops !
-        
+
         # store selection and deselect all
         plist = []
         for s in gpl.active.active_frame.strokes:
             for p in s.points:
                 plist.append([p, p.select])
                 p.select = False
-        
+
         # select
         ## foreach_set does not update
         # gpl.active.active_frame.strokes[paint_id].points.foreach_set('select', [True]*len(gpl.active.active_frame.strokes[paint_id].points))
         for p in gpl.active.active_frame.strokes[paint_id].points:
             p.select = True
-        
+
         # assign
         bpy.ops.object.mode_set(mode='EDIT_GPENCIL')
         vg = assign_vg(obj, vg_name)
@@ -131,7 +164,7 @@ def view_cage(obj):
         # restore
         for pl in plist:
             pl[0].select = pl[1]
-        
+
 
     ## View axis Mode ---
 
@@ -186,7 +219,7 @@ def view_cage(obj):
     ## Master (root) collection
     bpy.context.scene.collection.objects.link(cage)
 
-    # spawn cage and align it to view (Again ! align something to a vector !!! argg)
+    # spawn cage and align it to view
 
     r3d = bpy.context.space_data.region_3d
     viewmat = r3d.view_matrix
@@ -202,50 +235,64 @@ def view_cage(obj):
     lattice.points_v = 2
     lattice.points_w = 1
 
-    lattice.interpolation_type_u = lattice_interp#'KEY_LINEAR'-'KEY_BSPLINE'
-    lattice.interpolation_type_v = lattice_interp#'KEY_LINEAR'-'KEY_BSPLINE'
-    lattice.interpolation_type_w = lattice_interp#'KEY_LINEAR'-'KEY_BSPLINE'
+    lattice.interpolation_type_u = lattice_interp #'KEY_LINEAR'-'KEY_BSPLINE'
+    lattice.interpolation_type_v = lattice_interp
+    lattice.interpolation_type_w = lattice_interp
 
     mod = obj.grease_pencil_modifiers.new('tmp_lattice', 'GP_LATTICE')
+    if from_obj:
+        mods = []
+        for o in other_gp:
+            mods.append( o.grease_pencil_modifiers.new('tmp_lattice', 'GP_LATTICE') )
 
     # move to top if modifiers exists
     for _ in range(len(obj.grease_pencil_modifiers)):
         bpy.ops.object.gpencil_modifier_move_up(modifier='tmp_lattice')
+    if from_obj:
+        for o in other_gp:
+            for _ in range(len(o.grease_pencil_modifiers)):
+                bpy.ops.object.gpencil_modifier_move_up({'object':o}, modifier='tmp_lattice')
 
     mod.object = cage
+    if from_obj:
+        for m in mods:
+            m.object = cage
 
     if initial_mode == 'PAINT_GPENCIL':
         mod.layer = gpl.active.info
-    
+
     # note : if initial was Paint, changed to Edit
     #        so vertex attribution is valid even for paint
     if bpy.context.mode == 'EDIT_GPENCIL':
         mod.vertex_group = vg.name
 
-    #Go in object mode if not already
+    # Go in object mode if not already
     if bpy.context.mode != 'OBJECT':
         bpy.ops.object.mode_set(mode='OBJECT')
 
-    # Store name of deformed object in case of 'revive modal' 
+    # Store name of deformed object in case of 'revive modal'
     cage.vertex_groups.new(name=obj.name)
+    if from_obj:
+        for o in other_gp:
+            cage.vertex_groups.new(name=o.name)
 
     ## select and make cage active
     # cage.select_set(True)
     bpy.context.view_layer.objects.active = cage
-    obj.select_set(False)#deselect GP object
-    bpy.ops.object.mode_set(mode='EDIT')# go in lattice edit mode
-    bpy.ops.lattice.select_all(action='SELECT')# select all points
+    obj.select_set(False) # deselect GP object
+    bpy.ops.object.mode_set(mode='EDIT') # go in lattice edit mode
+    bpy.ops.lattice.select_all(action='SELECT') # select all points
 
     if prefs.use_clic_drag:
         ## Eventually change tool mode to tweak for direct point editing (reset after before leaving)
-        bpy.ops.wm.tool_set_by_id(name="builtin.select")# Tweaktoolcode
+        bpy.ops.wm.tool_set_by_id(name="builtin.select") # Tweaktoolcode
     return cage
 
 
 def back_to_obj(obj, gp_mode, org_lattice_toolset, context):
-    if context.mode == 'EDIT_LATTICE' and org_lattice_toolset:# Tweaktoolcode - restore the active tool used by lattice edit..
-        bpy.ops.wm.tool_set_by_id(name = org_lattice_toolset)# Tweaktoolcode
-    
+    if context.mode == 'EDIT_LATTICE' and org_lattice_toolset: # Tweaktoolcode - restore the active tool used by lattice edit..
+        bpy.ops.wm.tool_set_by_id(name = org_lattice_toolset) # Tweaktoolcode
+
     # gp object active and selected
     bpy.ops.object.mode_set(mode='OBJECT')
     obj.select_set(True)
@@ -257,25 +304,45 @@ def delete_cage(cage):
     bpy.data.objects.remove(cage)
     bpy.data.lattices.remove(lattice)
 
-def apply_cage(gp_obj, cage):
+def apply_cage(gp_obj):
     mod = gp_obj.grease_pencil_modifiers.get('tmp_lattice')
+    multi_user = None
     if mod:
-        bpy.ops.object.gpencil_modifier_apply(apply_as='DATA', modifier=mod.name)
+        if gp_obj.data.users > 1:
+            old = gp_obj.data
+            multi_user = old.name
+            other_user = [o for o in bpy.data.objects if o is not gp_obj and o.data is old]
+            gp_obj.data = gp_obj.data.copy()
+
+        # bpy.ops.object.gpencil_modifier_apply(apply_as='DATA', modifier=mod.name)
+        bpy.ops.object.gpencil_modifier_apply({'object': gp_obj}, apply_as='DATA', modifier=mod.name)
+
+        if multi_user:
+            for o in other_user: # relink
+                o.data = gp_obj.data
+            bpy.data.grease_pencils.remove(old)
+            gp_obj.data.name = multi_user
+
     else:
         print('tmp_lattice modifier not found to apply...')
 
-    delete_cage(cage)
-
-def cancel_cage(gp_obj, cage):
+def cancel_cage(self):
     #remove modifier
-    mod = gp_obj.grease_pencil_modifiers.get('tmp_lattice')
+    mod = self.gp_obj.grease_pencil_modifiers.get('tmp_lattice')
     if mod:
-        gp_obj.grease_pencil_modifiers.remove(mod)
+        self.gp_obj.grease_pencil_modifiers.remove(mod)
     else:
-        print('tmp_lattice modifier not found to remove...')
-    
-    delete_cage(cage)
+        print(f'tmp_lattice modifier not found to remove on {self.gp_obj.name}')
     
+    for ob in self.other_gp:
+        mod = ob.grease_pencil_modifiers.get('tmp_lattice')
+        if mod:
+            ob.grease_pencil_modifiers.remove(mod)
+        else:
+            print(f'tmp_lattice modifier not found to remove on {ob.name}')
+
+    delete_cage(self.cage)
+
 
 class GP_OT_latticeGpDeform(bpy.types.Operator):
     """Create a lattice to use as quad corner transform"""
@@ -302,7 +369,7 @@ valid:Spacebar/Enter, cancel:Del/Backspace/Tab/Ctrl+T"
         if event.type in {'Z'} and event.value == 'PRESS' and event.ctrl:
             ## Disable (capture key)
             return {"RUNNING_MODAL"}
-            ## Not found how possible to find modal start point in undo stack to 
+            ## Not found how possible to find modal start point in undo stack to
             # print('ops list', context.window_manager.operators.keys())
             # if context.window_manager.operators:#can be empty
             #     print('\nlast name', context.window_manager.operators[-1].name)
@@ -320,7 +387,7 @@ valid:Spacebar/Enter, cancel:Del/Backspace/Tab/Ctrl+T"
         if event.type in {'H'} and event.value == 'PRESS':
             # self.report({'INFO'}, "Can't hide")
             return {"RUNNING_MODAL"}
-        
+
         if event.type in {'ONE'} and event.value == 'PRESS':# , 'NUMPAD_1'
             self.lat.points_u = self.lat.points_v = 2
             return {"RUNNING_MODAL"}
@@ -361,25 +428,25 @@ valid:Spacebar/Enter, cancel:Del/Backspace/Tab/Ctrl+T"
             self.lat.points_u = 2
             self.lat.points_v = 1
             return {"RUNNING_MODAL"}
-        
+
         if event.type in {'RIGHT_ARROW'} and event.value == 'PRESS' and event.ctrl:
             if self.lat.points_u < 20:
-                self.lat.points_u += 1 
+                self.lat.points_u += 1
             return {"RUNNING_MODAL"}
 
         if event.type in {'LEFT_ARROW'} and event.value == 'PRESS' and event.ctrl:
             if self.lat.points_u > 1:
-                self.lat.points_u -= 1 
+                self.lat.points_u -= 1
             return {"RUNNING_MODAL"}
 
         if event.type in {'UP_ARROW'} and event.value == 'PRESS' and event.ctrl:
             if self.lat.points_v < 20:
-                self.lat.points_v += 1 
+                self.lat.points_v += 1
             return {"RUNNING_MODAL"}
 
         if event.type in {'DOWN_ARROW'} and event.value == 'PRESS' and event.ctrl:
             if self.lat.points_v > 1:
-                self.lat.points_v -= 1 
+                self.lat.points_v -= 1
             return {"RUNNING_MODAL"}
 
 
@@ -391,21 +458,29 @@ valid:Spacebar/Enter, cancel:Del/Backspace/Tab/Ctrl+T"
             return {"RUNNING_MODAL"}
 
         # Valid
-        if event.type in {'RET', 'SPACE'}:
+        if event.type in {'RET', 'SPACE', 'NUMPAD_ENTER'}:
             if event.value == 'PRESS':
                 context.window_manager.boxdeform_running = False
                 self.restore_prefs(context)
                 back_to_obj(self.gp_obj, self.gp_mode, self.org_lattice_toolset, context)
-                apply_cage(self.gp_obj, self.cage)#must be in object mode
-                
-                # back to original mode 
+                if event.shift:
+                    # Let the cage as is with a unique ID
+                    store_cage(self, 'lattice_cage_deform_group')
+                else:
+                    apply_cage(self.gp_obj) # must be in object mode
+                    assign_vg(self.gp_obj, 'lattice_cage_deform_group', delete=True)
+                    for o in self.other_gp:
+                        apply_cage(o)
+                        assign_vg(o, 'lattice_cage_deform_group', delete=True)
+                    delete_cage(self.cage)
+
+                # back to original mode
                 if self.gp_mode != 'OBJECT':
                     bpy.ops.object.mode_set(mode=self.gp_mode)
-
                 context.area.header_text_set(None)#reset header
 
                 return {'FINISHED'}
-        
+
         # Abort ---
         # One Warning for Tab cancellation.
         if event.type == 'TAB' and event.value == 'PRESS':
@@ -431,15 +506,16 @@ valid:Spacebar/Enter, cancel:Del/Backspace/Tab/Ctrl+T"
         context.window_manager.boxdeform_running = False
         self.restore_prefs(context)
         back_to_obj(self.gp_obj, self.gp_mode, self.org_lattice_toolset, context)
-        cancel_cage(self.gp_obj, self.cage)
-        context.area.header_text_set(None)     
+        cancel_cage(self)
+        assign_vg(self.gp_obj, 'lattice_cage_deform_group', delete=True)
+        context.area.header_text_set(None)
         if self.gp_mode != 'OBJECT':
             bpy.ops.object.mode_set(mode=self.gp_mode)
 
     def store_prefs(self, context):
         # store_valierables <-< preferences
-        self.use_drag_immediately = context.preferences.inputs.use_drag_immediately 
-        self.drag_threshold_mouse = context.preferences.inputs.drag_threshold_mouse 
+        self.use_drag_immediately = context.preferences.inputs.use_drag_immediately
+        self.drag_threshold_mouse = context.preferences.inputs.drag_threshold_mouse
         self.drag_threshold_tablet = context.preferences.inputs.drag_threshold_tablet
         self.use_overlays = context.space_data.overlay.show_overlays
         # maybe store in windows manager to keep around in case of modal revival ?
@@ -450,7 +526,7 @@ valid:Spacebar/Enter, cancel:Del/Backspace/Tab/Ctrl+T"
         context.preferences.inputs.drag_threshold_mouse = self.drag_threshold_mouse
         context.preferences.inputs.drag_threshold_tablet = self.drag_threshold_tablet
         context.space_data.overlay.show_overlays = self.use_overlays
-    
+
     def set_prefs(self, context):
         context.preferences.inputs.use_drag_immediately = True
         context.preferences.inputs.drag_threshold_mouse = 1
@@ -475,8 +551,8 @@ valid:Spacebar/Enter, cancel:Del/Backspace/Tab/Ctrl+T"
         self.org_lattice_toolset = None
         ## usability toggles
         if self.prefs.use_clic_drag:#Store the active tool since we will change it
-            self.org_lattice_toolset = bpy.context.workspace.tools.from_space_view3d_mode(bpy.context.mode, create=False).idname# Tweaktoolcode    
-        
+            self.org_lattice_toolset = bpy.context.workspace.tools.from_space_view3d_mode(bpy.context.mode, create=False).idname# Tweaktoolcode
+
         #store (scene properties needed in case of ctrlZ revival)
         self.store_prefs(context)
         self.gp_mode = 'EDIT_GPENCIL'
@@ -502,19 +578,26 @@ valid:Spacebar/Enter, cancel:Del/Backspace/Tab/Ctrl+T"
 
         if context.object.type != 'GPENCIL':
             # self.report({'ERROR'}, "Works only on gpencil objects")
-            ## silent return 
+            ## silent return
             return {'CANCELLED'}
 
-        #paint need VG workaround. object need good shortcut
         if context.mode not in ('EDIT_GPENCIL', 'OBJECT', 'PAINT_GPENCIL'):
-            # self.report({'WARNING'}, "Works only in following GPencil modes: edit")# ERROR
-            ## silent return 
+            # self.report({'WARNING'}, "Works only in following GPencil modes: object / edit/ paint")# ERROR
+            ## silent return
             return {'CANCELLED'}
 
+            
         # bpy.ops.ed.undo_push(message="Box deform step")#don't work as expected (+ might be obsolete)
-        # https://developer.blender.org/D6147 <- undo forget 
+        # https://developer.blender.org/D6147 <- undo forget
 
         self.gp_obj = context.object
+
+        self.from_object = context.mode == 'OBJECT'
+        self.all_gps = self.other_gp = []
+        if self.from_object:
+            self.all_gps = [o for o in bpy.context.selected_objects if o.type == 'GPENCIL']
+            self.other_gp = [o for o in self.all_gps if o is not self.gp_obj]
+
         # Clean potential failed previous job (delete tmp lattice)
         mod = self.gp_obj.grease_pencil_modifiers.get('tmp_lattice')
         if mod:
@@ -526,13 +609,13 @@ valid:Spacebar/Enter, cancel:Del/Backspace/Tab/Ctrl+T"
             print('Deleted remaining lattice object')
             delete_cage(phantom_obj)
 
-        if [m for m in self.gp_obj.grease_pencil_modifiers if m.type == 'GP_LATTICE']:
-            self.report({'ERROR'}, "Grease pencil object already has a lattice modifier (can only have one)")
-            return {'CANCELLED'}
-        
+        if bpy.app.version < (2,93,0):
+            if [m for m in self.gp_obj.grease_pencil_modifiers if m.type == 'GP_LATTICE']:
+                self.report({'ERROR'}, "Grease pencil object already has a lattice modifier (multi-lattices are enabled in blender 2.93+)")
+                return {'CANCELLED'}
+
+        self.gp_mode = context.mode # store mode for restore
 
-        self.gp_mode = context.mode#store mode for restore
-        
         # All good, create lattice and start modal
 
         # Create lattice (and switch to lattice edit) ----
@@ -540,7 +623,7 @@ valid:Spacebar/Enter, cancel:Del/Backspace/Tab/Ctrl+T"
         if isinstance(self.cage, str):#error, cage not created, display error
             self.report({'ERROR'}, self.cage)
             return {'CANCELLED'}
-        
+
         self.lat = self.cage.data
 
         self.set_prefs(context)
@@ -581,4 +664,4 @@ def unregister():
     wm = bpy.context.window_manager
     p = 'boxdeform_running'
     if p in wm:
-        del wm[p]
\ No newline at end of file
+        del wm[p]
