@@ -40,16 +40,13 @@ from .internals import (
     update_collection_tree,
     update_property_group,
     generate_state,
+    check_state,
     get_move_selection,
     get_move_active,
     update_qcd_header,
+    add_vertical_separator_line,
 )
 
-from .qcd_operators import (
-    QCDAllBase,
-)
-
-
 preview_collections = {}
 last_icon_theme_text = None
 last_icon_theme_text_sel = None
@@ -79,6 +76,7 @@ class CollectionManager(Operator):
         cm = context.scene.collection_manager
         prefs = context.preferences.addons[__package__].preferences
         view_layer = context.view_layer
+        collection = context.view_layer.layer_collection.collection
 
         if view_layer.name != cls.last_view_layer:
             if prefs.enable_qcd:
@@ -136,12 +134,22 @@ class CollectionManager(Operator):
             renum_sec.alignment = 'LEFT'
             renum_sec.operator("view3d.renumerate_qcd_slots")
 
+        undo_sec = op_sec.row(align=True)
+        undo_sec.alignment = 'LEFT'
+        undo_sec.operator("view3d.undo_wrapper", text="", icon='LOOP_BACK')
+        undo_sec.operator("view3d.redo_wrapper", text="", icon='LOOP_FORWARDS')
+
         # menu & filter
         right_sec = button_row_1.row()
         right_sec.alignment = 'RIGHT'
 
-        right_sec.menu("VIEW3D_MT_CM_specials_menu")
-        right_sec.popover(panel="COLLECTIONMANAGER_PT_display_options",
+        specials_menu = right_sec.row()
+        specials_menu.alignment = 'RIGHT'
+        specials_menu.menu("VIEW3D_MT_CM_specials_menu")
+
+        display_options = right_sec.row()
+        display_options.alignment = 'RIGHT'
+        display_options.popover(panel="COLLECTIONMANAGER_PT_display_options",
                            text="", icon='FILTER')
 
         mc_box = layout.box()
@@ -157,21 +165,79 @@ class CollectionManager(Operator):
         prop = c_icon.operator("view3d.set_active_collection",
                                               text='', icon='GROUP', depress=highlight)
         prop.is_master_collection = True
-        prop.collection_name = 'Master Collection'
+        prop.collection_name = 'Scene Collection'
 
         master_collection_row.separator()
 
         # name
-        name_row = master_collection_row.row()
-        name_row.prop(self, "master_collection", text='')
-        name_row.enabled = False
+        name_row = master_collection_row.row(align=True)
+        name_field = name_row.row(align=True)
+        name_field.prop(self, "master_collection", text='')
+        name_field.enabled = False
+
+        # set selection
+        setsel = name_row.row(align=True)
+        icon = 'DOT'
+        some_selected = False
+
+        if not collection.objects:
+            icon = 'BLANK1'
+            setsel.active = False
+
+        else:
+            all_selected = None
+            all_unreachable = None
+
+            for obj in collection.objects:
+                if not obj.visible_get() or obj.hide_select:
+                    if all_unreachable != False:
+                        all_unreachable = True
+
+                else:
+                    all_unreachable = False
+
+                if obj.select_get() == False:
+                    # some objects remain unselected
+                    icon = 'KEYFRAME'
+                    all_selected = False
+
+                else:
+                    some_selected = True
+
+                    if all_selected == False:
+                        break
+
+                    all_selected = True
+
+
+            if all_selected:
+                # all objects are selected
+                icon = 'KEYFRAME_HLT'
+
+            if all_unreachable:
+                if collection.objects:
+                    icon = 'DOT'
+
+                setsel.active = False
+
+        prop = setsel.operator("view3d.select_collection_objects",
+                                   text="",
+                                   icon=icon,
+                                   depress=some_selected,
+                                   )
+        prop.is_master_collection = True
+        prop.collection_name = 'Scene Collection'
 
-        master_collection_row.separator()
 
         # global rtos
         global_rto_row = master_collection_row.row()
         global_rto_row.alignment = 'RIGHT'
 
+        # used as a separator (actual separator not wide enough)
+        global_rto_row.label()
+
+
+        # set collection
         row_setcol = global_rto_row.row()
         row_setcol.alignment = 'LEFT'
         row_setcol.operator_context = 'INVOKE_DEFAULT'
@@ -183,7 +249,10 @@ class CollectionManager(Operator):
 
         collection = context.view_layer.layer_collection.collection
 
-        icon = 'MESH_CUBE'
+        icon = 'IMPORT'
+
+        if collection.objects:
+            icon = 'MESH_CUBE'
 
         if selected_objects:
             if active_object and active_object.name in collection.objects:
@@ -193,12 +262,32 @@ class CollectionManager(Operator):
                 icon = 'STICKY_UVS_LOC'
 
         else:
-            row_setcol.enabled = False
+            row_setcol.active = False
 
-        prop = row_setcol.operator("view3d.set_collection", text="",
+
+        # add vertical separator line
+        separator = row_setcol.row()
+        separator.scale_x = 0.2
+        separator.enabled = False
+        separator.operator("view3d.cm_ui_separator_button",
+                            text="",
+                                icon='BLANK1',
+                                )
+
+        # add operator
+        prop = row_setcol.operator("view3d.send_objects_to_collection", text="",
                                    icon=icon, emboss=False)
         prop.is_master_collection = True
-        prop.collection_name = 'Master Collection'
+        prop.collection_name = 'Scene Collection'
+
+        # add vertical separator line
+        separator = row_setcol.row()
+        separator.scale_x = 0.2
+        separator.enabled = False
+        separator.operator("view3d.cm_ui_separator_button",
+                            text="",
+                                icon='BLANK1',
+                                )
 
         copy_icon = 'COPYDOWN'
         swap_icon = 'ARROW_LEFTRIGHT'
@@ -365,12 +454,16 @@ class CollectionManager(Operator):
 
         if cm.in_phantom_mode:
             view.enabled = False
+
             if prefs.enable_qcd:
                 renum_sec.enabled = False
 
+            undo_sec.enabled = False
+            specials_menu.enabled = False
+
             c_icon.enabled = False
             row_setcol.enabled = False
-            addcollec_row.enabled = False
+            button_row_2.enabled = False
 
 
     def execute(self, context):
@@ -387,58 +480,8 @@ class CollectionManager(Operator):
         if cm.cm_list_index >= len(cm.cm_list_collection):
             cm.cm_list_index = -1
 
-        # check if expanded & history/buffer state still correct
-        if internals.collection_state:
-            new_state = generate_state()
-
-            if new_state["name"] != internals.collection_state["name"]:
-                internals.copy_buffer["RTO"] = ""
-                internals.copy_buffer["values"].clear()
-
-                internals.swap_buffer["A"]["RTO"] = ""
-                internals.swap_buffer["A"]["values"].clear()
-                internals.swap_buffer["B"]["RTO"] = ""
-                internals.swap_buffer["B"]["values"].clear()
-
-                for name in list(internals.expanded):
-                    laycol = internals.layer_collections.get(name)
-                    if not laycol or not laycol["has_children"]:
-                        internals.expanded.remove(name)
-
-                for name in list(internals.expand_history["history"]):
-                    laycol = internals.layer_collections.get(name)
-                    if not laycol or not laycol["has_children"]:
-                        internals.expand_history["history"].remove(name)
-
-                for rto, history in internals.rto_history.items():
-                    if view_layer.name in history:
-                        del history[view_layer.name]
-
-
-            else:
-                for rto in ["exclude", "select", "hide", "disable", "render", "holdout", "indirect"]:
-                    if new_state[rto] != internals.collection_state[rto]:
-                        if view_layer.name in internals.rto_history[rto]:
-                            del internals.rto_history[rto][view_layer.name]
-
-                        if view_layer.name in internals.rto_history[rto+"_all"]:
-                            del internals.rto_history[rto+"_all"][view_layer.name]
-
-        # check if in phantom mode and if it's still viable
-        if cm.in_phantom_mode:
-            if internals.layer_collections.keys() != internals.phantom_history["initial_state"].keys():
-                cm.in_phantom_mode = False
-
-            if view_layer.name != internals.phantom_history["view_layer"]:
-                cm.in_phantom_mode = False
-
-            if not cm.in_phantom_mode:
-                for key, value in internals.phantom_history.items():
-                    try:
-                        value.clear()
-                    except AttributeError:
-                        if key == "view_layer":
-                            internals.phantom_history["view_layer"] = ""
+        # check if history/buffer/phantom state still correct
+        check_state(context, cm_popup=True, phantom_mode=True)
 
         # handle window sizing
         max_width = 960
@@ -497,14 +540,14 @@ class CM_UL_items(UIList):
     filter_by_selected: BoolProperty(
                         name="Filter By Selected",
                         default=False,
-                        description="Filter collections by selected items",
+                        description="Filter collections to only show the ones that contain the selected objects",
                         update=lambda self, context:
                             CM_UL_items.new_collections.clear(),
                         )
     filter_by_qcd: BoolProperty(
                         name="Filter By QCD",
                         default=False,
-                        description="Filter collections to only show QCD slots",
+                        description="Filter collections to only show the ones that are QCD slots",
                         update=lambda self, context:
                             CM_UL_items.new_collections.clear(),
                         )
@@ -583,7 +626,8 @@ class CM_UL_items(UIList):
             QCD.scale_x = 0.4
             QCD.prop(item, "qcd_slot_idx", text="")
 
-        c_name = row.row()
+        # collection name
+        c_name = row.row(align=True)
 
         #if rename[0] and index == cm.cm_list_index:
             #c_name.activate_init = True
@@ -591,16 +635,84 @@ class CM_UL_items(UIList):
 
         c_name.prop(item, "name", text="", expand=True)
 
+        # set selection
+        setsel = c_name.row(align=True)
+        icon = 'DOT'
+        some_selected = False
+
+        if not collection.objects:
+            icon = 'BLANK1'
+            setsel.active = False
+
+        if any((laycol["ptr"].exclude,
+               collection.hide_select,
+               collection.hide_viewport,
+               laycol["ptr"].hide_viewport,)):
+            # objects cannot be selected
+            setsel.active = False
+
+        else:
+            all_selected = None
+            all_unreachable = None
+
+            for obj in collection.objects:
+                if not obj.visible_get() or obj.hide_select:
+                    if all_unreachable != False:
+                        all_unreachable = True
+
+                else:
+                    all_unreachable = False
+
+                if obj.select_get() == False:
+                    # some objects remain unselected
+                    icon = 'KEYFRAME'
+                    all_selected = False
+
+                else:
+                    some_selected = True
+
+                    if all_selected == False:
+                        break
+
+                    all_selected = True
+
+
+            if all_selected:
+                # all objects are selected
+                icon = 'KEYFRAME_HLT'
+
+            if all_unreachable:
+                if collection.objects:
+                    icon = 'DOT'
+
+                setsel.active = False
+
+
+        prop = setsel.operator("view3d.select_collection_objects",
+                                   text="",
+                                   icon=icon,
+                                   depress=some_selected
+                                   )
+        prop.is_master_collection = False
+        prop.collection_name = item.name
+
         # used as a separator (actual separator not wide enough)
         row.label()
 
         row = s2 if cm.align_local_ops else s1
 
-        # add set_collection op
+
+        add_vertical_separator_line(row)
+
+
+        # add send_objects_to_collection op
         set_obj_col = row.row()
         set_obj_col.operator_context = 'INVOKE_DEFAULT'
 
-        icon = 'MESH_CUBE'
+        icon = 'IMPORT'
+
+        if collection.objects:
+            icon = 'MESH_CUBE'
 
         if selected_objects:
             if active_object and active_object.name in collection.objects:
@@ -613,11 +725,13 @@ class CM_UL_items(UIList):
             set_obj_col.enabled = False
 
 
-        prop = set_obj_col.operator("view3d.set_collection", text="",
+        prop = set_obj_col.operator("view3d.send_objects_to_collection", text="",
                                    icon=icon, emboss=False)
         prop.is_master_collection = False
         prop.collection_name = item.name
 
+        add_vertical_separator_line(row)
+
 
         if cm.show_exclude:
             exclude_history_base = internals.rto_history["exclude"].get(view_layer.name, {})
@@ -749,12 +863,10 @@ class CM_UL_items(UIList):
 
         subrow = row.row(align=True)
         subrow.prop(self, "filter_name", text="")
-
-        icon = 'ZOOM_OUT' if self.use_filter_invert else 'ZOOM_IN'
-        subrow.prop(self, "use_filter_invert", text="", icon=icon)
+        subrow.prop(self, "use_filter_invert", text="", icon='ARROW_LEFTRIGHT')
 
         subrow = row.row(align=True)
-        subrow.prop(self, "filter_by_selected", text="", icon='SNAP_VOLUME')
+        subrow.prop(self, "filter_by_selected", text="", icon='STICKY_UVS_LOC')
 
         if context.preferences.addons[__package__].preferences.enable_qcd:
             subrow.prop(self, "filter_by_qcd", text="", icon='EVENT_Q')
@@ -892,6 +1004,10 @@ class SpecialsMenu(Menu):
                                text="Purge All Collections Without Objects")
         prop.without_objects = True
 
+        layout.separator()
+
+        layout.operator("view3d.select_all_cumulative_objects")
+
 
 class EnableAllQCDSlotsMenu(Menu):
     bl_label = "Global QCD Slot Actions"
@@ -900,6 +1016,10 @@ class EnableAllQCDSlotsMenu(Menu):
     def draw(self, context):
         layout = self.layout
 
+        layout.operator("view3d.create_all_qcd_slots")
+
+        layout.separator()
+
         layout.operator("view3d.enable_all_qcd_slots")
         layout.operator("view3d.enable_all_qcd_slots_isolated")
 
@@ -930,17 +1050,7 @@ def view3d_header_qcd_slots(self, context):
     layout = self.layout
     idx = 1
 
-    if internals.qcd_collection_state:
-        view_layer = context.view_layer
-        new_state = generate_state(qcd=True)
-
-        if (new_state["name"] != internals.qcd_collection_state["name"]
-        or  new_state["exclude"] != internals.qcd_collection_state["exclude"]
-        or  new_state["qcd"] != internals.qcd_collection_state["qcd"]):
-            if view_layer.name in internals.qcd_history:
-                del internals.qcd_history[view_layer.name]
-                internals.qcd_collection_state.clear()
-                QCDAllBase.clear()
+    check_state(context, qcd=True)
 
 
     main_row = layout.row(align=True)
@@ -994,7 +1104,8 @@ def view3d_header_qcd_slots(self, context):
             prop.slot = str(x+1)
 
         else:
-            row.label(text="", icon='X')
+            prop = row.operator("view3d.unassigned_qcd_slot", text="", icon='X', emboss=False)
+            prop.slot = str(x+1)
 
 
         if idx%5==0:
