@@ -20,9 +20,9 @@
 
 bl_info = {
     "name": "Rigify",
-    "version": (0, 6, 2),
+    "version": (0, 6, 4),
     "author": "Nathan Vegdahl, Lucio Rossi, Ivan Cappiello, Alexander Gavrilov",
-    "blender": (2, 82, 0),
+    "blender": (3, 0, 0),
     "description": "Automatic rigging from building-block components",
     "location": "Armature properties, Bone properties, View3d tools panel, Armature Add menu",
     "doc_url": "{BLENDER_MANUAL_URL}/addons/rigging/rigify/index.html",
@@ -64,9 +64,9 @@ initial_load_order = [
     'rig_ui_template',
     'generate',
     'rot_mode',
+    'operators',
     'ui',
 ]
-utils_module_name = __name__ + '.utils'
 
 
 def get_loaded_modules():
@@ -84,7 +84,10 @@ def reload_modules():
         importlib.reload(sys.modules[name])
 
 def compare_module_list(a, b):
-    # Allow 'utils' to move around
+    # HACK: ignore the "utils" module when comparing module load orders,
+    # because it is inconsistent for reasons unknown.
+    # See rBAa918332cc3f821f5a70b1de53b65dd9ca596b093.
+    utils_module_name = __name__ + '.utils'
     a_copy = list(a)
     a_copy.remove(utils_module_name)
     b_copy = list(b)
@@ -106,16 +109,13 @@ def load_initial_modules():
     return load_list
 
 def load_rigs():
-    if not legacy_loaded:
-        rig_lists.get_internal_rigs()
-        metarig_menu.init_metarig_menu()
+    rig_lists.get_internal_rigs()
+    metarig_menu.init_metarig_menu()
 
 
 if "reload_list" in locals():
     reload_modules()
 else:
-    legacy_loaded = False
-
     load_list = load_initial_modules()
 
     from . import (base_rig, base_generate, rig_ui_template, feature_set_list, rig_lists, generate, ui, metarig_menu)
@@ -140,6 +140,11 @@ from bpy.props import (
 )
 
 
+def get_generator():
+    """Returns the currently active generator instance."""
+    return base_generate.BaseGenerator.instance
+
+
 class RigifyFeatureSets(bpy.types.PropertyGroup):
     name: bpy.props.StringProperty()
     module_name: bpy.props.StringProperty()
@@ -150,77 +155,13 @@ class RigifyPreferences(AddonPreferences):
     # when defining this in a submodule of a python package.
     bl_idname = __name__
 
-    def update_legacy(self, context):
-        global legacy_loaded, reload_list
-
-        if self.legacy_mode:
-            if legacy_loaded:    # already in legacy mode. needed when rigify is reloaded
-                return
-            else:
-                unregister()
-                reload_modules()
-
-                globals().pop('utils')
-                globals().pop('rig_lists')
-                globals().pop('generate')
-                globals().pop('ui')
-                globals().pop('metarig_menu')
-
-                from .legacy import utils, rig_lists, generate, ui, metarig_menu
-
-                print("ENTERING RIGIFY LEGACY\r\n")
-
-                legacy_loaded = True
-                reload_list += [ m.__name__ for m in [ legacy, utils, rig_lists, generate, ui, metarig_menu ] ]
-
-                globals()['utils'] = legacy.utils
-                globals()['rig_lists'] = legacy.rig_lists
-                globals()['generate'] = legacy.generate
-                globals()['ui'] = legacy.ui
-                globals()['metarig_menu'] = legacy.metarig_menu
-
-                register()
-
-        else:
-            unregister()
-
-            globals().pop('utils')
-            globals().pop('rig_lists')
-            globals().pop('generate')
-            globals().pop('ui')
-            globals().pop('metarig_menu')
-
-            from . import utils, rig_lists, generate, ui, metarig_menu
-
-            print("EXIT RIGIFY LEGACY\r\n")
-
-            globals()['utils'] = utils
-            globals()['rig_lists'] = rig_lists
-            globals()['generate'] = generate
-            globals()['ui'] = ui
-            globals()['metarig_menu'] = metarig_menu
-
-            legacy_loaded = False
-            reload_list = reload_list_init
-            reload_modules()
-
-            load_rigs()
-
-            register()
-
     def register_feature_sets(self, register):
         """Call register or unregister of external feature sets"""
-        if self.legacy_mode:
-            return
-
         for set_name in feature_set_list.get_installed_list():
             feature_set_list.call_register_function(set_name, register)
 
     def update_external_rigs(self, force=False):
         """Get external feature sets"""
-        if self.legacy_mode:
-            return
-
         set_list = feature_set_list.get_installed_list()
 
         # Update feature set list
@@ -243,25 +184,12 @@ class RigifyPreferences(AddonPreferences):
             # Re-register rig parameters
             register_rig_parameters()
 
-    legacy_mode: BoolProperty(
-        name='Legacy Mode',
-        description='When enabled the add-on will run in legacy mode using the old 2.76b feature set',
-        default=False,
-        update=update_legacy
-    )
-
     rigify_feature_sets: bpy.props.CollectionProperty(type=RigifyFeatureSets)
     active_feature_set_index: IntProperty()
 
     def draw(self, context):
         layout = self.layout
 
-        layout.prop(self, 'legacy_mode')
-
-        if self.legacy_mode:
-            return
-
-
         layout.label(text="Feature Sets:")
 
         layout.operator("wm.rigify_add_feature_set", text="Install Feature Set from File...", icon='FILEBROWSER')
@@ -455,16 +383,17 @@ class RigifyParameterValidator(object):
     def __getattr__(self, name):
         return getattr(self.__params, name)
 
-    def __setattr__(self, name, val):
+    def __setattr__(self, name, val_original):
         # allow __init__ to work correctly
         if hasattr(RigifyParameterValidator, name):
-            return object.__setattr__(self, name, val)
+            return object.__setattr__(self, name, val_original)
 
-        if not (isinstance(val, tuple) and callable(val[0]) and isinstance(val[1], dict)):
-            print("!!! RIGIFY RIG %s: INVALID DEFINITION FOR RIG PARAMETER %s: %r\n" % (self.__rig_name, name, val))
+        if not isinstance(val_original, bpy.props._PropertyDeferred):
+            print("!!! RIGIFY RIG %s: INVALID DEFINITION FOR RIG PARAMETER %s: %r\n" % (self.__rig_name, name, val_original))
             return
 
         # actually defining the property modifies the dictionary with new parameters, so copy it now
+        val = (val_original.function, val_original.keywords)
         new_def = (val[0], val[1].copy())
 
         if 'poll' in new_def[1]:
@@ -480,7 +409,7 @@ class RigifyParameterValidator(object):
         if val[0] != bpy.props.CollectionProperty:
             val[1]['update'] = update_callback(name)
 
-        setattr(self.__params, name, val)
+        setattr(self.__params, name, val_original)
         self.__prop_table[name] = (self.__rig_name, new_def)
 
 
@@ -526,6 +455,7 @@ def register():
     ui.register()
     feature_set_list.register()
     metarig_menu.register()
+    operators.register()
 
     # Classes.
     for cls in classes:
@@ -600,6 +530,10 @@ def register():
         description="Forces Rigify to delete and rebuild all the rig widgets. if unset, only missing widgets will be created",
         default=False)
 
+    bpy.types.Armature.rigify_mirror_widgets = BoolProperty(name="Mirror Widgets",
+        description="Make widgets for left and right side bones linked duplicates with negative X scale for the right side, based on bone name symmetry",
+        default=True)
+
     bpy.types.Armature.rigify_target_rig = PointerProperty(type=bpy.types.Object,
         name="Rigify Target Rig",
         description="Defines which rig to overwrite. If unset, a new one called 'rig' will be created",
@@ -609,6 +543,10 @@ def register():
         name="Rigify Target Rig UI",
         description="Defines the UI to overwrite. If unset, 'rig_ui.py' will be used")
 
+    bpy.types.Armature.rigify_finalize_script = PointerProperty(type=bpy.types.Text,
+        name="Finalize Script",
+        description="Run this script after generation to apply user-specific changes")
+
     bpy.types.Armature.rigify_rig_basename = StringProperty(name="Rigify Rig Name",
         description="Defines the name of the Rig. If unset, in 'new' mode 'rig' will be used, in 'overwrite' mode the target rig name will be used",
         default="")
@@ -617,42 +555,30 @@ def register():
         name="Transfer Only Selected",
         description="Transfer selected bones only", default=True)
 
-    # Update legacy on restart or reload.
-    if legacy_loaded or bpy.context.preferences.addons['rigify'].preferences.legacy_mode:
-        bpy.context.preferences.addons['rigify'].preferences.legacy_mode = True
-
-    bpy.context.preferences.addons['rigify'].preferences.register_feature_sets(True)
-    bpy.context.preferences.addons['rigify'].preferences.update_external_rigs()
+    bpy.context.preferences.addons[__package__].preferences.register_feature_sets(True)
+    bpy.context.preferences.addons[__package__].preferences.update_external_rigs()
 
     # Add rig parameters
     register_rig_parameters()
 
 
 def register_rig_parameters():
-    if bpy.context.preferences.addons['rigify'].preferences.legacy_mode:
-        for rig in rig_lists.rig_list:
-            r = utils.get_rig_type(rig)
-            try:
+    for rig in rig_lists.rigs:
+        rig_module = rig_lists.rigs[rig]['module']
+        rig_class = rig_module.Rig
+        r = rig_class if hasattr(rig_class, 'add_parameters') else rig_module
+        try:
+            if hasattr(r, 'add_parameters'):
                 r.add_parameters(RigifyParameterValidator(RigifyParameters, rig, RIGIFY_PARAMETER_TABLE))
-            except AttributeError:
-                pass
-    else:
-        for rig in rig_lists.rigs:
-            rig_module = rig_lists.rigs[rig]['module']
-            rig_class = rig_module.Rig
-            r = rig_class if hasattr(rig_class, 'add_parameters') else rig_module
-            try:
-                if hasattr(r, 'add_parameters'):
-                    r.add_parameters(RigifyParameterValidator(RigifyParameters, rig, RIGIFY_PARAMETER_TABLE))
-            except Exception:
-                import traceback
-                traceback.print_exc()
+        except Exception:
+            import traceback
+            traceback.print_exc()
 
 
 def unregister():
     from bpy.utils import unregister_class
 
-    bpy.context.preferences.addons['rigify'].preferences.register_feature_sets(False)
+    bpy.context.preferences.addons[__package__].preferences.register_feature_sets(False)
 
     # Properties on PoseBones and Armature.
     del bpy.types.PoseBone.rigify_type
@@ -686,6 +612,7 @@ def unregister():
     clear_rigify_parameters()
 
     # Sub-modules.
+    operators.unregister()
     metarig_menu.unregister()
     ui.unregister()
     feature_set_list.unregister()
