@@ -18,8 +18,6 @@
 
 # <pep8 compliant>
 
-# TODO, use PREFERENCES_OT_* prefix for operators.
-
 import bpy
 from bpy.types import (
     Operator,
@@ -36,63 +34,40 @@ from bpy.props import (
 from bpy.app.translations import pgettext_tip as tip_
 
 
-def module_filesystem_remove(path_base, module_name):
+def _zipfile_root_namelist(file_to_extract):
+    # Return a list of root paths from zipfile.ZipFile.namelist.
     import os
+    root_paths = []
+    for f in file_to_extract.namelist():
+        # Python's `zipfile` API always adds a separate at the end of directories.
+        # use `os.path.normpath` instead of `f.removesuffix(os.sep)`
+        # since paths could be stored as `./paths/./`.
+        #
+        # Note that `..` prefixed paths can exist in ZIP files but they don't write to parent directory when extracting.
+        # Nor do they pass the `os.sep not in f` test, this is important,
+        # otherwise `shutil.rmtree` below could made to remove directories outside the installation directory.
+        f = os.path.normpath(f)
+        if os.sep not in f:
+            root_paths.append(f)
+    return root_paths
+
+
+def _module_filesystem_remove(path_base, module_name):
+    # Remove all Python modules with `module_name` in `base_path`.
+    # The `module_name` is expected to be a result from `_zipfile_root_namelist`.
+    import os
+    import shutil
     module_name = os.path.splitext(module_name)[0]
     for f in os.listdir(path_base):
         f_base = os.path.splitext(f)[0]
         if f_base == module_name:
             f_full = os.path.join(path_base, f)
-
             if os.path.isdir(f_full):
-                os.rmdir(f_full)
+                shutil.rmtree(f_full)
             else:
                 os.remove(f_full)
 
 
-# This duplicates shutil.copytree from Python 3.8, with the new dirs_exist_ok
-# argument that we need. Once we upgrade to 3.8 we can remove this.
-def _preferences_copytree(entries, src, dst):
-    import os
-    import shutil
-    from shutil import Error
-
-    os.makedirs(dst, exist_ok=True)
-    errors = []
-
-    for srcentry in entries:
-        srcname = os.path.join(src, srcentry.name)
-        dstname = os.path.join(dst, srcentry.name)
-        srcobj = srcentry
-        try:
-            if srcentry.is_symlink():
-                linkto = os.readlink(srcname)
-                os.symlink(linkto, dstname)
-                shutil.copystat(srcobj, dstname, follow_symlinks=False)
-            elif srcentry.is_dir():
-                preferences_copytree(srcobj, dstname)
-            else:
-                shutil.copy2(srcentry, dstname)
-        except Error as err:
-            errors.extend(err.args[0])
-        except OSError as why:
-            errors.append((srcname, dstname, str(why)))
-    try:
-        shutil.copystat(src, dst)
-    except OSError as why:
-        if getattr(why, 'winerror', None) is None:
-            errors.append((src, dst, str(why)))
-    if errors:
-        raise Error(errors)
-    return dst
-
-
-def preferences_copytree(src, dst):
-    import os
-    with os.scandir(src) as entries:
-        return _preferences_copytree(entries=entries, src=src, dst=dst)
-
-
 class PREFERENCES_OT_keyconfig_activate(Operator):
     bl_idname = "preferences.keyconfig_activate"
     bl_label = "Activate Keyconfig"
@@ -115,7 +90,7 @@ class PREFERENCES_OT_copy_prev(Operator):
 
     @classmethod
     def _old_version_path(cls, version):
-        return bpy.utils.resource_path('USER', version[0], version[1])
+        return bpy.utils.resource_path('USER', major=version[0], minor=version[1])
 
     @classmethod
     def previous_version(cls):
@@ -124,6 +99,15 @@ class PREFERENCES_OT_copy_prev(Operator):
         version = bpy.app.version
         version_new = ((version[0] * 100) + version[1])
         version_old = ((version[0] * 100) + version[1]) - 1
+
+        # Special case, remove when the version is > 3.0.
+        if version_new == 300:
+            version_new = 294
+            version_old = 293
+        else:
+            print("TODO: remove exception!")
+        # End special case.
+
         # Ensure we only try to copy files from a point release.
         # The check below ensures the second numbers match.
         while (version_new % 100) // 10 == (version_old % 100) // 10:
@@ -168,10 +152,8 @@ class PREFERENCES_OT_copy_prev(Operator):
         return os.path.isfile(old_userpref) and not os.path.isfile(new_userpref)
 
     def execute(self, _context):
-        # Use this instead once we upgrade to Python 3.8 with dirs_exist_ok.
-        # import shutil
-        # shutil.copytree(self._old_path(), self._new_path(), dirs_exist_ok=True)
-        preferences_copytree(self._old_path(), self._new_path())
+        import shutil
+        shutil.copytree(self._old_path(), self._new_path(), dirs_exist_ok=True, symlinks=True)
 
         # reload preferences and recent-files.txt
         bpy.ops.wm.read_userpref()
@@ -244,7 +226,11 @@ class PREFERENCES_OT_keyconfig_import(Operator):
 
         config_name = basename(self.filepath)
 
-        path = bpy.utils.user_resource('SCRIPTS', os.path.join("presets", "keyconfig"), create=True)
+        path = bpy.utils.user_resource(
+            'SCRIPTS',
+            path=os.path.join("presets", "keyconfig"),
+            create=True,
+        )
         path = os.path.join(path, config_name)
 
         try:
@@ -282,7 +268,7 @@ class PREFERENCES_OT_keyconfig_export(Operator):
     )
     filepath: StringProperty(
         subtype='FILE_PATH',
-        default="keymap.py",
+        default="",
     )
     filter_folder: BoolProperty(
         name="Filter folders",
@@ -321,7 +307,13 @@ class PREFERENCES_OT_keyconfig_export(Operator):
         return {'FINISHED'}
 
     def invoke(self, context, _event):
+        import os
         wm = context.window_manager
+        if not self.filepath:
+            self.filepath = os.path.join(
+                os.path.expanduser("~"),
+                bpy.path.display_name_to_filepath(wm.keyconfigs.active.name) + ".py",
+            )
         wm.fileselect_add(self)
         return {'RUNNING_MODAL'}
 
@@ -547,7 +539,11 @@ class PREFERENCES_OT_theme_install(Operator):
 
         xmlfile = self.filepath
 
-        path_themes = bpy.utils.user_resource('SCRIPTS', "presets/interface_theme", create=True)
+        path_themes = bpy.utils.user_resource(
+            'SCRIPTS',
+            path=os.path.join("presets", "interface_theme"),
+            create=True,
+        )
 
         if not path_themes:
             self.report({'ERROR'}, "Failed to get themes path")
@@ -608,7 +604,7 @@ class PREFERENCES_OT_addon_install(Operator):
         name="Target Path",
         items=(
             ('DEFAULT', "Default", ""),
-            ('PREFS', "User Prefs", ""),
+            ('PREFS', "Preferences", ""),
         ),
     )
 
@@ -640,8 +636,8 @@ class PREFERENCES_OT_addon_install(Operator):
         pyfile = self.filepath
 
         if self.target == 'DEFAULT':
-            # don't use bpy.utils.script_paths("addons") because we may not be able to write to it.
-            path_addons = bpy.utils.user_resource('SCRIPTS', "addons", create=True)
+            # Don't use `bpy.utils.script_paths(path="addons")` because we may not be able to write to it.
+            path_addons = bpy.utils.user_resource('SCRIPTS', path="addons", create=True)
         else:
             path_addons = context.preferences.filepaths.script_directory
             if path_addons:
@@ -680,11 +676,12 @@ class PREFERENCES_OT_addon_install(Operator):
                 traceback.print_exc()
                 return {'CANCELLED'}
 
+            file_to_extract_root = _zipfile_root_namelist(file_to_extract)
             if self.overwrite:
-                for f in file_to_extract.namelist():
-                    module_filesystem_remove(path_addons, f)
+                for f in file_to_extract_root:
+                    _module_filesystem_remove(path_addons, f)
             else:
-                for f in file_to_extract.namelist():
+                for f in file_to_extract_root:
                     path_dest = os.path.join(path_addons, os.path.basename(f))
                     if os.path.exists(path_dest):
                         self.report({'WARNING'}, "File already installed to %r\n" % path_dest)
@@ -700,7 +697,7 @@ class PREFERENCES_OT_addon_install(Operator):
             path_dest = os.path.join(path_addons, os.path.basename(pyfile))
 
             if self.overwrite:
-                module_filesystem_remove(path_addons, os.path.basename(pyfile))
+                _module_filesystem_remove(path_addons, os.path.basename(pyfile))
             elif os.path.exists(path_dest):
                 self.report({'WARNING'}, "File already installed to %r\n" % path_dest)
                 return {'CANCELLED'}
@@ -899,7 +896,8 @@ class PREFERENCES_OT_app_template_install(Operator):
         filepath = self.filepath
 
         path_app_templates = bpy.utils.user_resource(
-            'SCRIPTS', os.path.join("startup", "bl_app_templates_user"),
+            'SCRIPTS',
+            path=os.path.join("startup", "bl_app_templates_user"),
             create=True,
         )
 
@@ -923,11 +921,12 @@ class PREFERENCES_OT_app_template_install(Operator):
                 traceback.print_exc()
                 return {'CANCELLED'}
 
+            file_to_extract_root = _zipfile_root_namelist(file_to_extract)
             if self.overwrite:
-                for f in file_to_extract.namelist():
-                    module_filesystem_remove(path_app_templates, f)
+                for f in file_to_extract_root:
+                    _module_filesystem_remove(path_app_templates, f)
             else:
-                for f in file_to_extract.namelist():
+                for f in file_to_extract_root:
                     path_dest = os.path.join(path_app_templates, os.path.basename(f))
                     if os.path.exists(path_dest):
                         self.report({'WARNING'}, "File already installed to %r\n" % path_dest)
@@ -1004,7 +1003,7 @@ class PREFERENCES_OT_studiolight_install(Operator):
         prefs = context.preferences
 
         path_studiolights = os.path.join("studiolights", self.type.lower())
-        path_studiolights = bpy.utils.user_resource('DATAFILES', path_studiolights, create=True)
+        path_studiolights = bpy.utils.user_resource('DATAFILES', path=path_studiolights, create=True)
         if not path_studiolights:
             self.report({'ERROR'}, "Failed to create Studio Light path")
             return {'CANCELLED'}
@@ -1042,7 +1041,7 @@ class PREFERENCES_OT_studiolight_new(Operator):
         default="StudioLight",
     )
 
-    ask_overide = False
+    ask_override = False
 
     def execute(self, context):
         import os
@@ -1050,15 +1049,19 @@ class PREFERENCES_OT_studiolight_new(Operator):
         wm = context.window_manager
         filename = bpy.path.ensure_ext(self.filename, ".sl")
 
-        path_studiolights = bpy.utils.user_resource('DATAFILES', os.path.join("studiolights", "studio"), create=True)
+        path_studiolights = bpy.utils.user_resource(
+            'DATAFILES',
+            path=os.path.join("studiolights", "studio"),
+            create=True,
+        )
         if not path_studiolights:
             self.report({'ERROR'}, "Failed to get Studio Light path")
             return {'CANCELLED'}
 
         filepath_final = os.path.join(path_studiolights, filename)
         if os.path.isfile(filepath_final):
-            if not self.ask_overide:
-                self.ask_overide = True
+            if not self.ask_override:
+                self.ask_override = True
                 return wm.invoke_props_dialog(self, width=320)
             else:
                 for studio_light in prefs.studio_lights:
@@ -1078,7 +1081,7 @@ class PREFERENCES_OT_studiolight_new(Operator):
 
     def draw(self, _context):
         layout = self.layout
-        if self.ask_overide:
+        if self.ask_override:
             layout.label(text="Warning, file already exists. Overwrite existing file?")
         else:
             layout.prop(self, "filename")
